const fs = require("fs");
const crypto = require("crypto");
import { formatDate } from "./utils";
import {
  uploadSliceImage,
  uploadFormalImage,
  uploadStudent,
  saveCollectLog
} from "../modules/client/api";
import db from "@/plugins/db";

/**
 * 文件上传
 * @param {Object} options 上传配置信息
 * @param {String} type 上传类型：formal=>原图，slice=>剪切图
 */
function toUploadImg(options, type) {
  const formData = new FormData();
  const filePath =
    type === "formal" ? options.formalImgPath : options.sliceImgPath;

  const buffer = fs.readFileSync(filePath);
  let fsHash = crypto.createHash("md5");
  fsHash.update(buffer);
  let md5 = fsHash.digest("hex");
  formData.append("md5", md5);

  const file = new File([buffer], options.examNumber + ".jpg");
  formData.append("file", file);
  formData.append("level", options.level);
  formData.append("scanUserId", options.clientUserId);

  return type === "formal"
    ? uploadFormalImage(options, formData, { headers: { md5 } })
    : uploadSliceImage(options, formData, { headers: { md5 } });
}

function toUploadStudent(options) {
  const datas = {
    subjectId: options.subjectId,
    examNumber: options.examNumber
  };
  return uploadStudent(datas);
}

function toSaveCollectLog(options) {
  const datas = {
    workId: options.examId,
    subjectId: options.subjectId,
    examNumber: options.examNumber,
    clientUsername: options.clientUsername,
    clientUserId: options.clientUserId,
    clientUserLoginTime: options.clientUserLoginTime,
    time: formatDate(),
    level: options.level,
    name: options.studentName,
    manual: options.isManual + ""
  };
  return saveCollectLog(datas);
}

/**
 * 获取文件的MD5
 * @param {String} source 文件路径
 */
// function getMD5(source) {
//   const buffer = fs.readFileSync(source);
//   let fsHash = crypto.createHash("md5");
//   fsHash.update(buffer);
//   return fsHash.digest("hex");
// }

class UploadTask {
  constructor({
    taskList,
    uploadSuccessCallback,
    uploadTaskOverCallback,
    uploadErrorCallBack
  }) {
    this.taskList = taskList;
    this.setT = "";
    this.taskRunning = false;
    this.uploadSuccessCallback = uploadSuccessCallback;
    this.uploadTaskOverCallback = uploadTaskOverCallback;
    this.uploadErrorCallBack = uploadErrorCallBack;

    this.startUploadTask();
  }

  addUploadTask(data) {
    this.taskList.push(data);
  }

  getCurTask() {
    return this.taskList.shift();
  }

  async startUploadTask() {
    this.taskRunning = true;
    setTimeout(() => {
      this.runUploadTask();
    }, 10);
  }

  async runUploadTask() {
    if (!this.taskList.length || !this.taskRunning) {
      this.overUploadTask();
      return;
    }
    const curTask = this.getCurTask();

    const uploadAll = [
      toUploadImg(curTask, "formal"),
      toUploadImg(curTask, "slice")
    ];
    let uploadResult = true;
    await Promise.all(uploadAll).catch(() => {
      uploadResult = false;
    });

    let updateStdRes, saveLogRes;
    if (uploadResult) {
      updateStdRes = await toUploadStudent(curTask).catch(() => {});
      saveLogRes = await toSaveCollectLog(curTask).catch(() => {});

      if (updateStdRes && saveLogRes) {
        // 更新paperId
        await db.updateLocalPaperId(curTask.id, updateStdRes.paperId);
        this.uploadSuccessCallback(curTask);
      }
    }
    // 待定：只要有一个接口出错，则提示上传失败！
    if (!uploadResult || !updateStdRes || !saveLogRes) {
      this.uploadErrorCallBack(curTask);
    }

    this.setT = setTimeout(() => {
      this.runUploadTask();
    }, 10);
  }

  overUploadTask() {
    this.taskRunning = false;
    if (this.setT) clearTimeout(this.setT);
    this.uploadTaskOverCallback && this.uploadTaskOverCallback();
  }
  stopUploadTask() {
    this.taskRunning = false;
    this.taskList = [];
  }
}

export default UploadTask;

export { toUploadImg };
